<?php
/*
 * Plugin Name: БИФИТ Касса
 * Plugin URI: www.bifit.com
 * Description: Онлайн-касса
 * Version: 1.0
 * Author: BIFIT
 * Author URI: www.bifit.com
 * License: GNU General Public License version 2 or later;
 * License URI: http://www.gnu.org/licenses/gpl-2.0.html
 */

require(ABSPATH . WPINC . '/pluggable.php');

use Bifit\F\BifitFunctions as bifitF;

final class BifitMainClass
{
	public $version = '1.0';

	private static $_instance = null;

	public static function instance()
	{
		if (is_null(self::$_instance)) {
			self::$_instance = new self();
		}

		return self::$_instance;
	}

	public function __construct()
	{
		$this->define('BIFIT_ABSPATH', plugin_dir_path( __FILE__));
		$this->define('BIFIT_ABSPATH_VIEWS', plugin_dir_path( __FILE__) . 'lib/views/');
		$this->includes();
		$this->load_options();
		$this->hooks();
		$this->wp_hooks();
	}

	public function wp_hooks()
	{
		register_activation_hook(__FILE__, array('setup_bifit', 'activation'));

		add_action('woocommerce_order_status_changed', array($this, 'wc_status_changed'), 10, 4);
		add_action('wpsc_purchase_log_save', array($this, 'wpec_status_changed'));
	}

	public function hooks()
	{
		add_action('bifit_settings_general', array($this, 'settings_general_html'));
		add_action('bifit_settings_woocommerce', array($this, 'settings_woocommerce_html'));
		add_action('bifit_settings_wpec', array($this, 'settings_wpec_html'));
	}

	public function includes()
	{
		require_once(BIFIT_ABSPATH . 'lib/setup.php');
		require_once(BIFIT_ABSPATH . 'lib/functions.php');

		if (is_admin())
		{
			require_once(BIFIT_ABSPATH . 'lib/admin.php');
			add_action('init', array('Bifit_Admin', 'init'));
		}
	}

	private function define($name, $value)
	{
		if (!defined($name)) {
			define($name, $value);
		}
	}

	public function load_options()
	{
		$this->admin 				= get_option('admin', 1);
		$this->test 				= get_option('test', 1);
		$this->log 					= get_option('log', 1);
		$this->token 				= get_option('token', '');
		$this->cashier_name 		= get_option('cashier_name', 'Иванов И.И.');
		$this->cashier_inn 			= get_option('cashier_inn', '');
		$this->pay_address 			= get_option('pay_address', 'https://site.ru');
		$this->address_type 		= get_option('address_type', 'email');
		$this->fio 					= get_option('fio', 0);
		$this->receipt_type 		= get_option('receipt_type', 'SALE');
		$this->tax_system 			= get_option('tax_system', 'COMMON');
		$this->calc_method 			= get_option('calc_method', 'FULL_PAY');
		$this->payment_type 		= get_option('payment_type', 'CARD');
		$this->product_vat			= get_option('product_vat', 'WITHOUT_VAT');
		$this->shipping_vat			= get_option('shipping_vat', 'WITHOUT_VAT');
		$this->woocommerce_status 	= json_decode(get_option('woocommerce_status'), true);
		$this->woocommerce_payment 	= json_decode(get_option('woocommerce_payment'), true);
		$this->wpec_status 			= get_option('wpec_status', '3');
		$this->wpec_payment 		= get_option('wpec_payment', array());

		if (empty($this->token) || $this->test == 1) {
			$this->token = 'P5cKbUUD9uSSrSlGdzspLblvBnD0GzTAE0cLmAPSEMxJ79DtLE';
		}
	}

	public function settings_general_html()
	{
		self::load_options();
		?>
		<table class="form-table">
			<tbody>
				<tr>
					<th scope="row">
						<label for="admin">Работа в админ-разделе:</label></th>
					<td><select name="admin" id="admin">
						<?php bifitF::dropdown_list(bifitF::dropdown_yesno_list(), $this->admin); ?>
						</select>
						<p class="description">Контролировать изменение статуса у сохранённых заказов?</p></td>
				</tr>
				<tr>
					<th scope="row">
						<label for="test">Тестовый режим:</label></th>
					<td><select name="test" id="test">
						<?php bifitF::dropdown_list(bifitF::dropdown_yesno_list(), $this->test); ?>
						</select>
						<p class="description">Выберите режим работы плагина...</p></td>
				</tr>
				<tr>
					<th scope="row">
						<label for="log">Логирование:</label></th>
					<td><select name="log" id="log">
						<?php bifitF::dropdown_list(bifitF::dropdown_yesno_list(), $this->log);	?>
						</select>
						<p class="description">Сохранять все запросы и ответы в отчётах?</p></td>
				</tr>
				<tr>
					<th scope="row">
						<label for="token">Токен доступа:</label></th>
					<td><input type="text" name="token" id="token" value="<?php echo esc_attr($this->token); ?>" class="regular-text">
						<p class="description">Введите выданный токен доступа...</p></td>
				</tr>
				<tr>
					<th scope="row">
						<label for="cashier_name">ФИО кассира:</label></th>
					<td><input type="text" name="cashier_name" id="cashier_name" value="<?php echo esc_attr($this->cashier_name); ?>" class="regular-text">
						<p class="description">Введите фамилию и инициалы кассира...</p></td>
				</tr>
				<tr>
					<th scope="row">
						<label for="cashier_inn">ИНН кассира:</label></th>
					<td><input type="text" name="cashier_inn" id="cashier_inn" value="<?php echo esc_attr($this->cashier_inn); ?>" class="regular-text">
						<p class="description">Введите 12-ти значный идентификационный номер налогоплательщика...</p></td>
				</tr>
				<tr>
					<th scope="row">
						<label for="pay_address">Место расчётов:</label></th>
					<td><input type="text" name="pay_address" id="pay_address" value="<?php echo esc_attr($this->pay_address); ?>" class="regular-text">
						<p class="description">Адрес сайта обязательно с HTTP или HTTPS протоколом подлкючения...</p></td>
				</tr>
				<tr>
					<th scope="row"><label for="address_type">Адрес клиента:</label></th>
					<td><select name="address_type" id="address_type">
						<?php bifitF::dropdown_list(bifitF::dropdown_address_type_list(), $this->address_type); ?>
						</select>
						<p class="description">Выберите данные клиента для вывода в чеке...</p></td>
				</tr>
				<tr>
					<th scope="row">
						<label for="fio">ФИО клиента:</label></th>
					<td><select name="fio" id="fio">
						<?php bifitF::dropdown_list(bifitF::dropdown_yesno_list(), $this->fio);	?>
						</select>
						<p class="description">Нужно ли выводить ФИО клиента в чеке?</p></td>
				</tr>
				<tr>
					<th scope="row"><label for="receipt_type">Тип документа:</label></th>
					<td><select name="receipt_type" id="receipt_type">
						<?php bifitF::dropdown_list(bifitF::dropdown_receipt_type_list(), $this->receipt_type); ?>
						</select></td>
				</tr>
				<tr>
					<th scope="row"><label for="tax_system">Система налогообложения:</label></th>
					<td><select name="tax_system" id="tax_system">
						<?php bifitF::dropdown_list(bifitF::dropdown_tax_system_list(), $this->tax_system); ?>
						</select></td>
				</tr>
				<tr>
					<th scope="row"><label for="calc_method">Признак способа расчёта:</label></th>
					<td><select name="calc_method" id="calc_method">
						<?php bifitF::dropdown_list(bifitF::dropdown_calc_method_list(), $this->calc_method); ?>
						</select></td>
				</tr>
				<tr>
					<th scope="row"><label for="payment_type">Тип оплаты:</label></th>
					<td><select name="payment_type" id="payment_type">
						<?php bifitF::dropdown_list(bifitF::dropdown_payment_type_list(), $this->payment_type); ?>
						</select></td>
				</tr>
				<tr>
					<th scope="row"><label for="product_vat">НДС на товары:</label></th>
					<td><select name="product_vat" id="product_vat">
						<?php bifitF::dropdown_list(bifitF::dropdown_product_vat_list(), $this->product_vat); ?>
						</select></td>
				</tr>
				<tr>
					<th scope="row"><label for="shipping_vat">НДС на доставку:</label></th>
					<td><select name="shipping_vat" id="shipping_vat">
						<?php bifitF::dropdown_list(bifitF::dropdown_shipping_vat_list(), $this->shipping_vat); ?>
						</select></td>
				</tr>
			</tbody>
		</table>
		<?php
	}

	public function dropdown_woocommerce_status_list($value)
	{
		$list = wc_get_order_statuses();
		bifitF::dropdown_list($list, $value);
	}

	public function dropdown_woocommerce_payment_list($value)
	{
		$payment_gateways = WC()->payment_gateways->payment_gateways();
		$list = array();
		foreach ($payment_gateways as $gateway): 
			$list[$gateway->id] = $gateway->get_title();
		endforeach;
		bifitF::dropdown_list($list, $value);
	}	

	public function settings_woocommerce_html()
	{
		?>
		<table class="form-table">
			<tbody>
				<tr>
					<th scope="row"><label for="woocommerce_status">Статус оплаты:</label></th>
					<td><select name="woocommerce_status[]" id="woocommerce_status" multiple="multiple" size="7"><?php $this->dropdown_woocommerce_status_list($this->woocommerce_status); ?></select>
						<p class="description">Выберите статус(ы) оплаты для активации плагина...</p></td>
				</tr>
				<tr>
					<th scope="row"><label for="woocommerce_payment">Платёжные системы:</label></th>
					<td><select name="woocommerce_payment[]" id="woocommerce_payment" multiple="multiple" size="7"><?php $this->dropdown_woocommerce_payment_list($this->woocommerce_payment); ?></select>
						<p class="description">Выберите способ(ы) оплаты для активации плагина...</p></td>
				</tr>
			</tbody>
		</table>
		<?php
	}

	public function dropdown_wpec_status_list($value)
	{
		global $wpsc_purchlog_statuses;
		$list = array();
		foreach ($wpsc_purchlog_statuses as $status): 
			$list[$status['order']] = $status['label'];
		endforeach;
		bifitF::dropdown_list($list, $value);
	}

	public function dropdown_wpec_payment_list($value)
	{
		$gateways = apply_filters('wpsc_settings_get_gateways', array());
		$list = array();
		foreach ($gateways as $gateway): 
			$list[$gateway['id']] = esc_html($gateway['name']);
		endforeach;
		bifitF::dropdown_list($list, $value);
	}

	public function settings_wpec_html()
	{
		?>
		<table class="form-table">
			<tbody>
				<tr>
					<th scope="row"><label for="wpec_status">Статус оплаты:</label></th>
					<td><select name="wpec_status[]" id="wpec_status" multiple="multiple" size="7">
						<?php $this->dropdown_wpec_status_list($this->wpec_status); ?>
						</select>
						<p class="description">Выберите статус(ы) оплаты для активации плагина...</p></td>
				</tr>
				<tr>
					<th scope="row"><label for="wpec_payment">Платёжные системы:</label></th>
					<td><select name="wpec_payment[]" id="wpec_payment" multiple="multiple" size="7">
						<?php $this->dropdown_wpec_payment_list($this->wpec_payment); ?>
						</select>
						<p class="description">Выберите способ(ы) оплаты для активации плагина...</p></td>
				</tr>
			</tbody>
		</table>
		<?php
	}

	public static function getProdAttr($product_id, $attr)
	{
		$a = 'pa_' . $attr;
		$d = wc_get_product_terms($product_id, $a);		

		$slug = $d[0]->slug;
		if ($slug) {
			return $slug;
		}

		return "";
	}

	public function __formatprc($price)
	{
		// конфигурация магазина
		$dec_count	= get_option('woocommerce_price_num_decimals', 2);

		// форматируем цену согласно настроек
		$price		= $dec_count < 0 ? round($price, $dec_count) : number_format($price, $dec_count, '.', '');

		return $price;
	}

	public function __checkInn($inn)
	{
		// только цифры в количестве 10 или 12 шт.
		$inn 		= (!preg_match("/^\d{12}$/", $inn)) ? 'ИНН задан в неверном формате!' : $inn;

		return $inn;
	}

	public function __checkPhone($tel)
	{
		// удаляем лишние символы
		$phone 		= str_replace(array('+', '(', ')', '-', ' '), '', trim($tel));

		// получаем первую цифру
		$first 		= substr($phone, "0", 1);

		// если номер из 10 цифр и первая цифра не 7, то добавляем 7
		$phone 		= strlen($phone)==10 && $first != 7 ? '7' . $phone : $phone;

		// окончательная проверка
		$phone 		= (!preg_match("/^[0-9]{11}+$/", $phone)) ? 'Телефон задан в неверном формате!' : $phone;

		return $phone;
	}

	public function __checkEmail($email)
	{
		// проверка модели записи
		$email 		= (!preg_match("/^\+\d+$|^\S+@\S+$/", $email)) ? 'E-mail задан в неверном формате!' : $email;

		return $email;
	}

	public function __checkProductPrice($item_price, $order_subtotal, $order_discount = 0)
	{
		// если скидка больше нуля
		if ($order_discount > 0)
		{
			// вычисляем процент скидки
			$percent 	= $order_discount * 100 / $order_subtotal;

			// уменьшаем цену товара согласно скидки
			$item_price = $item_price - ($item_price * ($percent / 100));
		}

		return $item_price;
	}

	public function __loadData($url, $headers = false, $data = false)
	{
		// инициализируем сеанс cURL
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);

		// добавляем данные
		if ($data)
		{
			curl_setopt($ch, CURLOPT_POST, true);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
		}

		// возврат результата в качестве строки
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

		// добавляем headers
		if ($headers)
		{
			curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
		}

		// получаем ответ
		$output	= curl_exec($ch);

		// завершаем сеанс cURL
		curl_close($ch);
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  
		return $output;
	}

	public function __saveReport($test, $bifit, $receipt, $receipt_id, $doc, $logObj)
	{
		global $wpdb;

		// выводим место и свойство операции
		$log  	= '####### Боевой запрос' . $loc . "\n";
		$log  	= $test != '' ? '####### Тестовый запрос' . "\n" : $log;

		// устанавливаем часовой пояс согласно настройкам сайта
		date_default_timezone_set(get_option('timezone_string'));
		$log   .= '####### ' . date('d-m-Y H:i:s', time()) . "\n";

		// ОТВЕТ #1

		$log   .= '####### Ответ #1: ' . "\n";

		// данные ответа #1
		if (is_object($bifit))
		{
			foreach ($bifit as $key=>$val)
			{
				$log .= $key . ': ' . $val . "\n";
			}
		}

		// ЧЕК

		// сформированный чек для отправки
		$log   .= '####### Чек: ' . "\n" . $receipt . "\n";

		// ОТВЕТ #2

		$log   .= '####### Ответ #2: ' . "\n";

		// данные ответа #2
		if (is_object($receipt_id))
		{
			foreach ($receipt_id as $key=>$val)
			{
				if (is_array($val))
				{
					foreach ($val as $k1=>$v1) 
					{
						if (is_object($val[$k1]))
						{
							foreach ($val[$k1] as $k2=>$v2) 
							{
								$log .= $v2 != '' ? ' – ' . $k2 . ': ' . $v2 . "\n" : '';
							}
						}
					}
				}
				else
				{
					$log .= $val != '' ? $key . ': ' . $val . "\n" : '';
				}
			}
		} 
		else 
		{
			$log   .= $receipt_id . "\n";
		}

		// ОТВЕТ #3

		$log   .= '####### Ответ #3: ' . "\n";
		$a_3 	= '';

		// данные ответа #3
		if (is_object($doc))
		{
			foreach ($doc as $key=>$val)
			{
				if (is_object($val))
				{
					foreach ($val as $k1=>$v1)
					{
						if (is_array($v1))
						{
							foreach ($v1 as $k2=>$v2) 
							{
								if (is_object($v1[$k2]))
								{
									$a_3 .= "\n";
									foreach ($v1[$k2] as $k3=>$v3) 
									{
										$a_3 .= $v3 != '' ? '   - ' . $k3 . ': ' . $v3 . "\n" : '';
									}
								}
							}
							$a_3 .= "\n";
						}
						elseif (is_object($v1))
						{
							foreach ($v1 as $k2=>$v2) 
							{
								if (is_object($v2))
								{
									foreach ($v2 as $k3=>$v3) 
									{
										$a_3 .= $v3 != '' ? '   - ' . $k3 . ': ' . $v3 . "\n" : '';
									}
								}
								else
								{
									$a_3 .= $v2 != '' ? '  - ' . $k2 . ': ' . $v2 . "\n" : '';
								}
							}
						}
						else
						{
							$a_3 .= $v1 != '' ? ' - ' . $k1 . ': ' . $v1 . "\n" : '';
						}
					}
				} 
				else 
				{
					$a_3 .= $val != '' ? $key . ': ' . $val . "\n" : '';
				}
			}
		}

		$log   .= $a_3;

		// ССЫЛКА НА ЧЕК

		// сформированная ссылка на чек
		$log   .= '####### Ссылка на чек: ' . "\n" . $logObj->receipt_link;

		// СОХРАНЯЕМ В БД

		// параметры объекта
		$logObj->doc = $a_3 != '' ? $a_3 : '';
		$logObj->log = $log;

		// сохранение
		//$this->writeLog('request', $logObj);
		$wpdb->insert(
			$wpdb->prefix . 'bifit_reports', 
			array(
				'order_id' 		=> $logObj->order_id, 
				'order_number' 	=> $logObj->order_number, 
				'order_date' 	=> $logObj->order_date, 
				'cashier' 		=> $logObj->cashier, 
				'client' 		=> $logObj->client, 
				'receipt_link' 	=> $logObj->receipt_link, 
				'receipt_id' 	=> $logObj->receipt_id, 
				'receipt_type' 	=> $logObj->receipt_type, 
				'payment_type' 	=> $logObj->payment_type, 
				'doc' 			=> $logObj->doc, 
				'log' 			=> $logObj->log
			)
		);
	}

	public function __mainProcess($order, $items, $test, $token, $log_ok)
	{
		// 1. АВТОРИЗАЦИЯ
		//    процесс необходимый для подключения клиента к фискальному процессингу

		// адрес для POST-запроса
		$url 		= 'https://fp' . $test . '.bifit.com/processing-api/oauth/token';

		// данные ссылки
		$data		= 'token=' . $token;
		$data  	   .= '&client_id=processing-connector-token';
		$data  	   .= '&client_secret=processing-connector-token';
		$data 	   .= '&grant_type=token';

		// headers
		$headers 	= array('Content-Type: application/x-www-form-urlencoded');

		// декодируем полученный ответ
		$bifit 	 	= json_decode(self::__loadData($url, $headers, $data));

		// 2. ФОРМИРОВАНИЕ КОНТЕНТА ЧЕКА
		//    формирование информации для дальнейшей отправки на фискализацию
		
		$order_data = $order->get_data();

		// параметры плагина
		$cashier_name 		= $this->cashier_name;
		$cashier_inn 		= $this->cashier_inn;
		$receipt_type 		= $this->receipt_type;
		$tax_system 		= $this->tax_system;
		$payment_type 		= $this->payment_type;
		$payment_address 	= $this->pay_address;
		$address_type 		= $this->address_type;
		$fio	 			= $this->fio;

		// адрес клиента в чеке
		if ($address_type == 'phone' && $order_data['billing']['phone']) {
			$address = self::__checkPhone($order_data['billing']['phone']);
		} else {
			$address = self::__checkEmail($order_data['billing']['email']);
		}

		// новый массив
		$r_products	= array();

		foreach ($items as $key_id=>$item)
		{
			// получаем данные товара
			$item_data = $item->get_data();

			// назначаем НДС
			if ($this->product_vat == 70) {
				$vat = $this->getProdAttr($item->get_product()->get_ID(), "stavka_nds");
			} else {
				$vat = $this->product_vat;
			}

			// назначаем признак способа расчёта
			if ($this->calc_method == 70) {
				$calc_method = $this->getProdAttr($item->get_product()->get_ID(), "priznak_sposoba_rascheta");
			} else {
				$calc_method = $this->calc_method;
			}

			// цена товара
			$price_piece = round(($item_data['total'] + $item_data['total_tax']) / $item_data['quantity'], 2);

			// очищаем имя товара от спец-символов
			$p_name = preg_replace('/[^\p{L}0-9 ]/iu', ' ', html_entity_decode($item_data['name']));
			$p_name = preg_replace('/ +/', ' ', trim($p_name));

			// формируем массив товаров для чека
			$r_products[$key_id]						= new stdClass();
			$r_products[$key_id]->product_name 			= $p_name;
			$r_products[$key_id]->product_item_price	= $price_piece;
			$r_products[$key_id]->product_quantity 		= $item_data['quantity'];
			$r_products[$key_id]->product_tax 			= $vat;
			$r_products[$key_id]->calc_method 			= $calc_method;
		}

		$items = '';

		// формируем данные чека
		foreach ($r_products as $key_id=>$product)
		{
			// выводим данные товара в чек
			$items .= $items != '' ? ', ' : '';
			$items .= '{';
			$items .= '"calculationMethod":"' . $product->calc_method . '", ';
			$items .= '"paymentSubject":"PRODUCT", ';
			$items .= '"name":"' . $product->product_name . '", ';
			$items .= '"price":' . $product->product_item_price . ', ';
			$items .= '"quantity":' . $product->product_quantity . ', ';
			$items .= '"vat":"' . $product->product_tax . '", ';
			$items .= '"total":' . ($product->product_item_price * $product->product_quantity);
			$items .= '}';
		}

		// доставка
		if ($order_data['shipping_total'] > 0)
		{
			// назначаем признак способа расчёта
			$calc_method_dost = $this->calc_method;
			if ($calc_method_dost == 70) {
				$calc_method_dost = 'FULL_PAY';
			}

			// выводим данные способа доставки в чек
			$items .= $items != '' ? ', ' : '';
			$items .= '{';
			$items .= '"calculationMethod":"' . $calc_method_dost . '", ';
			$items .= '"paymentSubject":"SERVICE", ';
			$items .= '"name":"Доставка", ';
			$items .= '"price":' . self::__formatprc($order_data['shipping_total']) . ', ';
			$items .= '"quantity":1, ';
			$items .= '"vat":"' . $this->shipping_vat . '", ';
			$items .= '"total":' . self::__formatprc($order_data['shipping_total'] + $order_data['shipping_tax']);
			$items .= '}';
		}

		// формируем чек
		$receipt  	= '{';
		$receipt   .= '"type":"' . $receipt_type . '", "taxSystem":"' . $tax_system . '", ';
		$receipt   .= '"cashier":{';
		$receipt   .= '"name":"' . $cashier_name . '"';
		$receipt   .= $cashier_inn != '' ? ', "inn":"' . self::__checkInn($cashier_inn) . '"' : '';
		$receipt   .= '}, ';
		$receipt   .= '"client":{';
		$receipt   .= $fio ? '"name":"' . trim($order_data['billing']['last_name'] . ' ' . $order_data['billing']['first_name']) . '", ' : '';
		$receipt   .= '"address":"' . $address . '"';
		$receipt   .= '}, ';
		$receipt   .= '"paymentAddress":"' . $payment_address . '", ';
		$receipt   .= '"items":[' . $items . '], ';
		$receipt   .= '"total":' . self::__formatprc($order_data["total"]) . ', ';
		$receipt   .= '"payments":{"' . $payment_type . '": ' . self::__formatprc($order_data["total"]) . '}';
		$receipt   .= '}';

		// 3. ОТПРАВКА ЧЕКА В ФИСКАЛЬНЫЙ ПРОЦЕССИНГ
		//    передача контента чека в фискальный процессинг для дальнейшей фискализации

		// адрес для POST-запроса
		$url 		= 'https://fp' . $test . '.bifit.com/processing-api/protected/documents/registration/receipts';

		// данные чека
		$data 		= $receipt;

		// ключ идемпотентности (Idempotency-Key)
		$o_number 	= $order_data['id'];
		$o_summ 	= self::__formatprc($order_data["total"]);
		date_default_timezone_set(get_option('timezone_string'));
		$date_time 	= date('Y-m-d H:i:s', time());
		$idemp_key 	= hash('sha256', "$o_number/$o_summ/$date_time");

		// headers
		$headers 	= array('Content-Type: application/json', 'Authorization: Bearer ' . $bifit->access_token, 'Idempotency-Key: ' . $idemp_key);

		// получаем идентификатор документа на процессинге (id)
		$receipt_id = json_decode(self::__loadData($url, $headers, $data));

		// 4. ПОЛУЧЕНИЕ ДОКУМЕНТА ПО ID
		//    получение документа с фискальными признаками по его номеру

		// параметры
		$doc 		= '';

		if (!is_object($receipt_id))
		{
			// ссылка для GET-запроса
			$url 	 = 'https://fp' . $test . '.bifit.com/processing-api/protected/documents/' . $receipt_id;

			// headers
			$headers = array('Authorization: Bearer ' . $bifit->access_token);

			// получаем документ
			$doc	 = json_decode(self::__loadData($url, $headers, false));
		}

		// ССЫЛКА НА ЧЕК

		$receipt_link 	= 'https://fp' . $test . '.bifit.com/processing-api/receipts/' . $idemp_key;

		// ЛОГ ОПЕРАЦИИ

		// Если включено логирование
		if ($log_ok)
		{
			// параметры для лога
			$logObj 				= new stdClass();
			$logObj->order_id 		= $order_data['id'];
			$logObj->order_number 	= $order_data['id'];
			$logObj->order_date 	= $date_time;
			$logObj->cashier 		= $cashier_inn != '' ? $cashier_name . ', ИНН:' . self::__checkInn($cashier_inn) : $cashier_name;
			$logObj->client 		= $fio ? trim($order_data['billing']['last_name'] . ' ' . $order_data['billing']['first_name']) . ', ' . $address : $address;
			$logObj->receipt_link 	= $receipt_link;
			$logObj->receipt_id 	= !is_object($receipt_id) ? $receipt_id : '000';
			$logObj->receipt_type 	= $receipt_type;
			$logObj->payment_type 	= $payment_type;
			$logObj->doc 			= '';
			$logObj->log 			= '';

			// сохраняем лог операции
			self::__saveReport($test, $bifit, $receipt, $receipt_id, $doc, $logObj);
		}
	}

	public function wpec_status_changed($order)
	{
		global $wpdb;

		require_once(WPSC_FILE_PATH . '/wpsc-includes/purchaselogs-items.class.php');

		$order_data = $order->get_data();

		if (in_array($order_data['processed'], $this->wpec_status) && in_array($order_data['gateway'], $this->wpec_payment))
		{
			if (is_admin() && !$this->admin) {
				return;
			}

			$order->init_items();
			$items	= $order->get_items();

			$test  	= $this->test == 1 ? '-test' : '';
			$token 	= $this->token;
			$log_ok = $this->log;

			self::__mainProcess($order, $items, $test, $token, $log_ok);
		}
	}

	public function wc_status_changed($id, $from, $to, $order)
	{
		global $wpdb;

		$order_data = $order->get_data();

		if (in_array('wc-' . $to, $this->woocommerce_status) && in_array($order_data['payment_method'], $this->woocommerce_payment))
		{
			if (is_admin() && !$this->admin) {
				return;
			}

			$items 	= $order_data['line_items'];

			$test  	= $this->test == 1 ? '-test' : '';
			$token 	= $this->token;
			$log_ok = $this->log;

			self::__mainProcess($order, $items, $test, $token, $log_ok);
		}
	}
}

function Bifit()
{
	return BifitMainClass::instance();
}

$GLOBALS['bifit'] = Bifit();
